Exporter subclass: #FrameworkExporter instanceVariableNames: 'outStream logStream outputFileNameWithSources outputFileNameWithoutSources loggerFilePath partialClassCategoryName filesToAppendFirst filesToAppendLast filesToExport strippedSourceCodeFilePath sourceStream'
	classVariableNames: ''!
FrameworkExporter category: 'Compiler-FileOut'.
FrameworkExporter comment: 'FrameworkExporter is a tool to export a framework.'.

!FrameworkExporter methodsFor: #export!
export: aClass init: aBoolean
	" Return the result of exporting definition and source code of a class. "

	| stream |
	stream := '' writeStream.
	self exportDefinitionOf: aClass on: stream.
	self exportMetaDefinitionOf: aClass on: stream.
	aBoolean ifTrue: [ stream nextPutAll: (self initCodeFor: aClass); cr].
	self exportMethodsOf: aClass on: stream.
	self exportMethodsOf: aClass class on: stream.
	^stream contents! !

!FrameworkExporter methodsFor: #initialize-release!
initialize
	super initialize.
	methodsForClassDictionary := Dictionary new.
	outputFileNameWithSources := nil.
	outputFileNameWithoutSources := nil.
	loggerFilePath := nil.
	partialClassCategoryName := nil.
	filesToAppendFirst := Array new.
	filesToAppendLast := Array new.
	filesToExport := Array new.
	self resetStreams.
	^self! !

!FrameworkExporter methodsFor: #initialize-release!
resetRecollectionVariables
	methodsForClassDictionary := Dictionary new.
	self resetStreams.
	^self! !

!FrameworkExporter methodsFor: #initialize-release!
resetStreams
	outStream := '' writeStream.
	logStream := '' writeStream.
	sourceStream := '' writeStream! !

!FrameworkExporter methodsFor: #fileOut!
collectClassesModifiedWhenEvaluate: aBlockClosure
	|createdClassNames|
	createdClassNames := Array new.
	self resetRecollectionVariables.	
	
	SystemManager current
		when: #methodBound:to:
		do: [:aMethod :aBehavior |
				aBehavior name = #DoIt ifFalse:[ |behaviorName behaviorValidatesInclusion|
					behaviorName := aBehavior instanceClass name.

					behaviorValidatesInclusion := partialClassCategoryName isNil or: [partialClassCategoryName isNil not and:[aBehavior instanceClass category includes: partialClassCategoryName]].
					
					(behaviorValidatesInclusion and:[(createdClassNames includes: behaviorName) not])
						ifTrue:[|methodsToExportCollection|
								methodsToExportCollection := methodsForClassDictionary at: behaviorName ifAbsentPut: [Array new].
								self appendLogForMethodCollect: aMethod.
								methodsToExportCollection add: aMethod]
						ifFalse: [(createdClassNames includes: behaviorName) 
									ifTrue: [methodsForClassDictionary at: behaviorName ifAbsentPut: [Array new]]]]];
		when: #classChanged:
		do: [:aBehavior | |behaviorName|
					behaviorName := aBehavior basicAt: '$className'.
					createdClassNames add: behaviorName.
					
					(partialClassCategoryName isNil
								or:[behaviorName isNil not and: [behaviorName includes: partialClassCategoryName]])
						ifTrue: [methodsForClassDictionary at: behaviorName put: Array new.
								logStream
									nextPutAll: '%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21' unescaped; cr;
									tab; nextPutAll: 'Class created: ', behaviorName; cr;
									nextPutAll: '%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21%21' unescaped; cr]].

	aBlockClosure value.

	SystemManager current
		removeActionsForEvent: #methodBound:to:;
		removeActionsForEvent: #classChanged:! !

!FrameworkExporter methodsFor: #fileOut!
exportClassesToFile: aFilePath

	filesToAppendFirst isEmpty
		ifFalse: [self appendFilesFromCollection: filesToAppendFirst].

	[methodsForClassDictionary
		keysAndValuesDo:
		[:aClassName :anArrayOfMethods | |aBehavior|
			aBehavior := smalltalk basicAt: aClassName.
			logStream nextPutAll: '*********************************************'; cr.
			
			anArrayOfMethods isEmpty
			ifTrue: [outStream nextPutAll: (self export: aBehavior init: true); cr.
					logStream tab; nextPutAll: '--- Full class exported%21%21%21 ---' unescaped; cr]
				ifFalse: [anArrayOfMethods
								do:[:aMethod |
									self export: aMethod classField method: aMethod on: outStream.
									logStream tab; nextPutAll: ('+++ Exported Method: ', aMethod toString); cr]].
			logStream
				tab; nextPutAll: ('--- class exported: ', aClassName); cr;
				nextPutAll: '*********************************************'; cr].

	filesToAppendLast isEmpty
		ifFalse: [self appendFilesFromCollection: filesToAppendLast].	
	
	outStream contents outputToFile: aFilePath.
	self appendLogForWrittenFile: aFilePath]
		on: Error
		do: [:anError | self appendLogForError: anError]! !

!FrameworkExporter methodsFor: #accessing!
export
	self exportSourcesModifiedWhenEvaluate: [filesToExport do: [:aFilePath | self importFileFrom: aFilePath]]! !

!FrameworkExporter methodsFor: #fileOut!
exportSourcesModifiedWhenEvaluate: aImportSourcesBlock
	| collectMethodsLogContent|
	collectMethodsLogContent := nil.	
	self resetRecollectionVariables.
	self collectClassesModifiedWhenEvaluate: aImportSourcesBlock.

	outputFileNameWithSources isNil
		ifFalse:[self  exportClassesToFile: outputFileNameWithSources.
				collectMethodsLogContent := logStream contents.
				loggerFilePath isNil ifFalse:[logStream contents outputToFile: ((self normalizeFilePath: loggerFilePath usingExtension: 'log'))]].
	
	(outputFileNameWithoutSources isNil not or: [strippedSourceCodeFilePath isNil not])
		ifTrue: [
			"Reseting streams"
			self resetStreams.
			
			collectMethodsLogContent isNil ifFalse:[logStream nextPutAll: collectMethodsLogContent].
			
			"Removing sources"
			methodsForClassDictionary
				keysAndValuesDo: [:aClassName :anArrayOfMethods | |aBehavior|
					aBehavior := smalltalk basicAt: aClassName.
					
					self extractCommentOf: aBehavior; removeCommentOf: aBehavior.
					
					anArrayOfMethods isEmpty
						ifTrue: [aBehavior methodsDo: [:method| self extractSourceFromMethod: method; removeSourceOfMethod: method]]
						ifFalse: [anArrayOfMethods do: [:method| self extractSourceFromMethod: method; removeSourceOfMethod: method]].
					
					aBehavior isClass
						ifTrue:[aBehavior class methodsDo: [:method| self extractSourceFromMethod: method; removeSourceOfMethod: method]]].

			"Exporting sourceless framework"
			(outputFileNameWithoutSources isNil not)
				ifTrue: [self exportClassesToFile: outputFileNameWithoutSources].

			"Exporting framework sources"
			(strippedSourceCodeFilePath isNil not)
				ifTrue: [sourceStream contents outputToFile: strippedSourceCodeFilePath].
			
			loggerFilePath isNil ifFalse:[logStream contents outputToFile: (self normalizeFilePath: loggerFilePath usingExtension: 'noSources.log')]]! !

!FrameworkExporter methodsFor: #log!
extractCommentOf: aBehavior
	sourceStream nextPutAll: (self smalltalkNameFor: aBehavior), '.comment_(', aBehavior comment escapedCode,');'; cr.
	logStream
		tab; nextPutAll: ('--- Comment extracted from: ' , aBehavior instanceClass printString); cr! !

!FrameworkExporter methodsFor: #log!
extractSourceFromMethod: aMethod
	sourceStream nextPutAll: aMethod classField instanceClass name, '.source_(', aMethod source escapedCode,');'; cr.
	aMethod source: nil.
	logStream
		tab; tab; nextPutAll: ('--- Sources extracted from: ' , aMethod toString) ; cr! !

!FrameworkExporter methodsFor: #log!
removeCommentOf: aBehavior
	aBehavior comment: nil.
	logStream
		tab; nextPutAll: ('--- Comment removed from: ' , aBehavior instanceClass printString); cr! !

!FrameworkExporter methodsFor: #log!
removeSourceOfMethod: aMethod
	aMethod source: nil.
	logStream
		tab; tab; nextPutAll: ('--- Sources removed from: ' , aMethod toString) ; cr! !

!FrameworkExporter methodsFor: #log!
appendLogForWrittenFile: aFilePath
	logStream
		nextPutAll: '/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\';cr;
		tab; nextPutAll: 'Written File: ', aFilePath; cr;
		nextPutAll: '/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\';cr! !

!FrameworkExporter methodsFor: #log!
appendLogForError: anError
	logStream
		nextPutAll: '********************** ERROR **********************';cr;
		nextPutAll: (logStream smalltalkErrorOf: anError) messageText;cr;
		nextPutAll: anError stack;cr;
		nextPutAll: '***************************************************';cr! !

!FrameworkExporter methodsFor: #log!
appendFileAppendedLogFromPath: aFilePath
	|fileContents|
	fileContents := # #read: aFilePath.
	outStream nextPutAll: fileContents; cr.
	logStream
		nextPutAll: ('-------------- File appended ', aFilePath, '. --------------'); cr;
		tab; nextPutAll: fileContents; cr;
		nextPutAll: ('-------------------------------------------------');cr! !

!FrameworkExporter methodsFor: #log!
appendLogForMethodCollect: aMethod
	logStream
		tab; nextPutAll: '+++ Method: ', aMethod toString;cr;
		tab; nextPutAll: '+++ Method category: ', aMethod category;cr.
	aMethod classField isMetaclass
		ifFalse: [logStream tab; nextPutAll: ('+++ Method''s class category: ' , aMethod classField instanceClass category); cr].
	logStream
		nextPutAll: '-------------------------------------------------'; cr! !

!FrameworkExporter methodsFor: #importing!
importFileFrom: aFilePath
	logStream nextPutAll: '/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\';cr.
	((aFilePath upToLast: '.st') size = (aFilePath size - 2))
		ifTrue: [# #fileIn: aFilePath]
		ifFalse: [# #fileInJS: aFilePath].
	logStream
		tab; nextPutAll: 'File Imported: ', aFilePath; cr;
		nextPutAll: '/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\/*\';cr;cr;cr! !

!FrameworkExporter methodsFor: #importing!
appendFilesFromCollection: aFilePathCollection
	aFilePathCollection isEmpty
		ifFalse:[ aFilePathCollection
					do: [:filePath | self appendFileAppendedLogFromPath: filePath]]! !

!FrameworkExporter methodsFor: #log!
normalizeFilePath: aFilePath usingExtension: aExtensionString
	| preExtensionString resultString fullExtensionString |
	fullExtensionString := '.' , aExtensionString.
	preExtensionString :=  aFilePath upToLast: fullExtensionString.
	resultString := aFilePath.

	(aFilePath = preExtensionString
		and: [((aFilePath size - 3) = preExtensionString size) not])
		ifFalse:[aFilePath, fullExtensionString].

	^resultString! !

!FrameworkExporter methodsFor: #accessing!
outputFileNameWithSources: aFilePath
	outputFileNameWithSources := aFilePath! !

!FrameworkExporter methodsFor: #accessing!
outputFileNameWithoutSources: aFilePath
	outputFileNameWithoutSources := aFilePath! !

!FrameworkExporter methodsFor: #accessing!
loggerFilePath: aFilePath
	loggerFilePath := aFilePath! !

!FrameworkExporter methodsFor: #accessing!
partialClassCategoryName: aString
	partialClassCategoryName := aString! !

!FrameworkExporter methodsFor: #accessing!
addFilePathToAppendLast: aFilePath
	filesToAppendLast add: aFilePath! !

!FrameworkExporter methodsFor: #accessing!
addFilePathToAppendFirst: aFilePath
	filesToAppendFirst add: aFilePath! !

!FrameworkExporter methodsFor: #accessing!
addFileToExport: aFilePath
	filesToExport add: aFilePath! !

!FrameworkExporter methodsFor: #accessing!
strippedSourceCodeFilePath: aFilePath
	strippedSourceCodeFilePath := aFilePath! !