"
This file install the info() function for console.


"
Object	subclass: #StInfo
	category: #info!
StInfo comment: '
	Install a message/function to show information about the system from shell commands.
	@2014 Alejandro Reimondo aleReimondo@smalltalking.net'!

! StInfo class methodsFor: #startup !
launch
	" Private - The system has been launched and start execution. "

	[StInfo new forSystem] installAs: #info! !

! StInfo methodsFor: #info !
forSystem
	" Private - Return the info of the system. "

	^'System (',Smalltalk current classes size,' classes)
categories: ',(self echo: Class categories sorted),'
',(self echo: MethodReferences new missingGlobals)! !

! StInfo methodsFor: #info !
forGlobal: aName
	" Private - Return the info of global aName. "

	| value |
	aName isEmpty ifTrue: [ ^self echo: (MethodReferences new globalReferences) ].
	value := Smalltalk at: aName ifAbsent: [
		^self echo: (MethodReferences new referencesOfGlobal: aName)
	].
	(value isKindOf: Behavior) ifTrue: [
		^self forClass: value instanceClass
	].
	^self echo: (MethodReferences new referencesOfGlobal: aName)! !

! StInfo methodsFor: #info !
forClass: aClass
	" Private - Return the info of global aName. "

	^(Exporter new fileOutDefinitionOf: aClass),'
--- class messages ---',(self echo: aClass class selectors sorted),'
---    messages    ---',(self echo: aClass selectors sorted)! !

! StInfo methodsFor: #info !
forCategory: aName
	" Private - Return the info of category aName. "

	^self echo: (MethodReferences new searchCategory: aName)! !

! StInfo methodsFor: #info !
forMessage: selector
	" Private - Return the info of message selector. "

	^self echo: (MethodReferences new referencesOf: selector)! !

! StInfo methodsFor: #info !
forSource: spec
	" Private - Return the info of source spec. "

	| target |
	target := spec doIt.
	^(target respondsTo: #source) ifTrue: [ target source ] ! !

! StInfo methodsFor: #output !
echo: anArray
	" Private - Return the output for anArray. "

	^anArray inject: '' into: [:total :each| total,'
	',each ] ! !

! Behavior methodsFor: #info !
info
	" Return information about the receiver. "

	^StInfo new forClass: self instanceClass! !

! String methodsFor: #info !
info
	" Return information about the target specified by the receiver. "

	self isEmpty ifTrue: [ ^# #info ].
	((self first = $ ) or: [ self first isUppercase ]) ifTrue: [
		^StInfo new forGlobal: self trimBlanks
	].
	self first = $# ifTrue: [
		^StInfo new forCategory: (self copyFrom: 2 to: self size) trimBlanks
	].
	self first = $? ifTrue: [
		^StInfo new forMessage: (self copyFrom: 2 to: self size) trimBlanks
	].
	self first = $= ifTrue: [
		^StInfo new forSource: (self copyFrom: 2 to: self size) trimBlanks
	].
	(self includes: $ ) ifFalse: [ ^StInfo new forMessage: self ].
	^StInfo new echo: (self asArrayOfSubstrings collect: [:each| each info ])! !

"Install the info() support"
self print: 'smalltalk.StInfo.launch();'.
StInfo launch.!